<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\MigrationAction;

use Dedi\SyliusMigrationPlugin\Config\MigrationConfig;
use Dedi\SyliusMigrationPlugin\Dto\MigrationOption;
use Dedi\SyliusMigrationPlugin\Service\Logger\MigrationLoggerInterface;
use Doctrine\DBAL\Connection;
use Psr\Log\LogLevel;

/**
 * Exemple de config :
 *
 *   truncate_table_before_migration:
 *     execute: true|false
 */
class CreateTempMappingTableBeforeMigration extends AbstractMigrationAction implements PreMigrationActionInterface
{
    private readonly Connection $connection;

    private readonly MigrationLoggerInterface $migrationLogger;

    private readonly MigrationConfig $migrationConfig;

    public function __construct(
        Connection $connection,
        MigrationLoggerInterface $migrationLogger,
        MigrationConfig $migrationConfig,
    ) {
        $this->connection = $connection;
        $this->migrationLogger = $migrationLogger;
        $this->migrationConfig = $migrationConfig;
    }

    public function getType(): string
    {
        return 'create_tmp_mapping_table_before_migration';
    }

    public function execute(MigrationOption $migrationOption): void
    {
        if (1 !== count($this->options) && true !== $this->options[0]) {
            return;
        }

        $sql = sprintf('CREATE TABLE IF NOT EXISTS `%s` (
          `id` INT NOT NULL AUTO_INCREMENT,
          `source_table_name` VARCHAR(255) NOT NULL,
          `source_column_name` VARCHAR(255) NOT NULL,
          `source_id` INT(11) NOT NULL,
          `dest_table_name` VARCHAR(255) NOT NULL,
          `dest_column_name` VARCHAR(255) NOT NULL,
          `dest_id`  INT(11) NOT NULL,
          PRIMARY KEY (`id`),
          INDEX `IDX_source_table_name` (`source_table_name` ASC),
          INDEX `IDX_source_column_name` (`source_column_name` ASC),
          INDEX `IDX_dest_table_name` (`dest_table_name` ASC),
          INDEX `IDX_dest_column_name` (`dest_column_name` ASC),
          INDEX `IDX_source_id` (`source_id` ASC),
          INDEX `IDX_dest_id` (`dest_id` ASC),
          INDEX `UNQ_source_to_dest_identifiers` (`source_table_name` ASC, `source_column_name` ASC, `source_id` ASC, `dest_table_name` ASC, `dest_column_name` ASC, `dest_id` ASC));
        ', $this->migrationConfig->getTempMappingTableName());

        $this->migrationLogger->log(LogLevel::INFO, $sql);
        if (!$migrationOption->isDryRun()) {
            $this->connection->executeQuery($sql);
        }
    }
}
