<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\MigrationAction;

use Dedi\SyliusMigrationPlugin\Dto\MigrationOption;

/**
 * Exemple de config :
 *
 *   pre_migration_executors: true|false
 */
class PreMigrationExecutorRegistry extends AbstractMigrationAction implements PreMigrationActionInterface
{
    /** @var PreMigrationExecutorInterface[] */
    private readonly iterable $executors;

    public function __construct(iterable $executors)
    {
        $this->executors = $executors;
    }

    public function getType(): string
    {
        return 'pre_migration_executors';
    }

    public function execute(MigrationOption $migrationOption): void
    {
        if (1 !== count($this->options) || 'true' !== $this->options[0]) {
            return;
        }

        foreach ($this->executors as $executor) {
            $executor->execute($migrationOption);
        }
    }
}
